@@ -4,7 +4,8 @@
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,7 +37,7 @@ typedef struct _grn_tokenizer_query {
   uint32_t size;
   grn_encoding encoding;
   uint32_t flags;
-  grn_bool have_tokenized_delimiter;
+  bool have_tokenized_delimiter;
   /* Deprecated since 4.0.8. Use tokenize_mode instead. */
   grn_token_mode token_mode;
   grn_tokenize_mode tokenize_mode;
@@ -48,8 +49,8 @@ typedef struct _grn_tokenizer_query {
   grn_id source_id;
   grn_obj *index_column;
   uint32_t normalize_flags;
-  grn_bool need_normalize;
-  grn_bool need_delimiter_check;
+  bool need_normalize;
+  bool need_delimiter_check;
 
   grn_obj *options;
 
