@@ -4,7 +4,8 @@
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -43,11 +44,11 @@ writer_write(mrb_state *mrb, mrb_value self)
     if (mrb_nil_p(target)) {
       GRN_OUTPUT_NULL();
     } else {
-      GRN_OUTPUT_BOOL(GRN_FALSE);
+      GRN_OUTPUT_BOOL(false);
     }
     break;
   case MRB_TT_TRUE :
-    GRN_OUTPUT_BOOL(GRN_TRUE);
+    GRN_OUTPUT_BOOL(true);
     break;
   case MRB_TT_INTEGER :
     GRN_OUTPUT_INT64(mrb_integer(target));
