@@ -1,10 +1,11 @@
 /*
   Copyright (C) 2015-2018  Brazil
-  Copyright (C) 2018-2023  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2018-2025  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -145,49 +146,59 @@ grn_obj_is_temporary(grn_ctx *ctx, grn_obj *obj)
   return false;
 }
 
-grn_bool
+bool
 grn_obj_is_builtin(grn_ctx *ctx, grn_obj *obj)
 {
   grn_id id;
 
   if (!obj) {
-    return GRN_FALSE;
+    return false;
   }
 
   id = grn_obj_id(ctx, obj);
   return grn_id_is_builtin(ctx, id);
 }
 
-grn_bool
+bool
 grn_obj_is_bulk(grn_ctx *ctx, grn_obj *obj)
 {
   if (!obj) {
-    return GRN_FALSE;
+    return false;
   }
 
   return obj->header.type == GRN_BULK;
 }
 
-grn_bool
+bool
 grn_obj_is_text_family_bulk(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_bulk(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   return GRN_TYPE_IS_TEXT_FAMILY(obj->header.domain);
 }
 
-grn_bool
+bool
 grn_obj_is_number_family_bulk(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_bulk(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   return grn_type_id_is_number_family(ctx, obj->header.domain);
 }
 
+bool
+grn_obj_maybe_record_bulk(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!grn_obj_is_bulk(ctx, obj)) {
+    return false;
+  }
+
+  return grn_id_maybe_table(ctx, obj->header.domain);
+}
+
 bool
 grn_obj_is_vector(grn_ctx *ctx, grn_obj *obj)
 {
@@ -202,12 +213,22 @@ bool
 grn_obj_is_text_family_vector(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_vector(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   return grn_type_id_is_text_family(ctx, obj->header.domain);
 }
 
+bool
+grn_obj_is_number_family_vector(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!grn_obj_is_uvector(ctx, obj)) {
+    return false;
+  }
+
+  return grn_type_id_is_number_family(ctx, obj->header.domain);
+}
+
 bool
 grn_obj_is_weight_vector(grn_ctx *ctx, grn_obj *obj)
 {
@@ -248,13 +269,13 @@ grn_obj_is_db(grn_ctx *ctx, grn_obj *obj)
   return obj->header.type == GRN_DB;
 }
 
-grn_bool
+bool
 grn_obj_is_table(grn_ctx *ctx, grn_obj *obj)
 {
-  grn_bool is_table = GRN_FALSE;
+  bool is_table = false;
 
   if (!obj) {
-    return GRN_FALSE;
+    return false;
   }
 
   switch (obj->header.type) {
@@ -262,7 +283,7 @@ grn_obj_is_table(grn_ctx *ctx, grn_obj *obj)
   case GRN_TABLE_HASH_KEY:
   case GRN_TABLE_PAT_KEY:
   case GRN_TABLE_DAT_KEY:
-    is_table = GRN_TRUE;
+    is_table = true;
     break;
   default:
     break;
@@ -274,7 +295,7 @@ grn_obj_is_table(grn_ctx *ctx, grn_obj *obj)
 bool
 grn_obj_is_table_with_key(grn_ctx *ctx, grn_obj *obj)
 {
-  grn_bool is_table_with_key = false;
+  bool is_table_with_key = false;
 
   if (!obj) {
     return false;
@@ -295,7 +316,7 @@ grn_obj_is_table_with_key(grn_ctx *ctx, grn_obj *obj)
 bool
 grn_obj_is_table_with_value(grn_ctx *ctx, grn_obj *obj)
 {
-  grn_bool is_table_with_value = false;
+  bool is_table_with_value = false;
 
   if (!obj) {
     return false;
@@ -433,20 +454,20 @@ grn_obj_is_result_set(grn_ctx *ctx, grn_obj *obj)
   return flags & GRN_OBJ_WITH_SUBREC;
 }
 
-grn_bool
+bool
 grn_obj_is_column(grn_ctx *ctx, grn_obj *obj)
 {
-  grn_bool is_column = GRN_FALSE;
+  bool is_column = false;
 
   if (!obj) {
-    return GRN_FALSE;
+    return false;
   }
 
   switch (obj->header.type) {
   case GRN_COLUMN_FIX_SIZE:
   case GRN_COLUMN_VAR_SIZE:
   case GRN_COLUMN_INDEX:
-    is_column = GRN_TRUE;
+    is_column = true;
   default:
     break;
   }
@@ -464,11 +485,11 @@ grn_obj_is_number_family_column(grn_ctx *ctx, grn_obj *obj)
   return grn_type_id_is_number_family(ctx, grn_obj_get_range(ctx, obj));
 }
 
-grn_bool
+bool
 grn_obj_is_scalar_column(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_column(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   return (obj->header.flags & GRN_OBJ_COLUMN_TYPE_MASK) ==
@@ -495,11 +516,11 @@ grn_obj_is_number_family_scalar_column(grn_ctx *ctx, grn_obj *obj)
   return grn_type_id_is_number_family(ctx, grn_obj_get_range(ctx, obj));
 }
 
-grn_bool
+bool
 grn_obj_is_vector_column(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_column(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   return (
@@ -517,11 +538,11 @@ grn_obj_is_text_family_vector_column(grn_ctx *ctx, grn_obj *obj)
   return grn_type_id_is_text_family(ctx, grn_obj_get_range(ctx, obj));
 }
 
-grn_bool
+bool
 grn_obj_is_weight_vector_column(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_vector_column(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   return (obj->header.flags & GRN_OBJ_WITH_WEIGHT) == GRN_OBJ_WITH_WEIGHT;
@@ -531,17 +552,17 @@ bool
 grn_obj_is_reference_column(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_column(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   const grn_id range_id = grn_obj_get_range(ctx, obj);
   if (grn_id_is_builtin(ctx, range_id)) {
-    return GRN_FALSE;
+    return false;
   }
 
   grn_obj *range = range = grn_ctx_at(ctx, range_id);
   if (!range) {
-    return GRN_FALSE;
+    return false;
   }
 
   uint8_t range_type = range->header.type;
@@ -558,66 +579,74 @@ grn_obj_is_reference_column(grn_ctx *ctx, grn_obj *obj)
   }
 }
 
-grn_bool
+bool
 grn_obj_is_data_column(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_column(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   return obj->header.type == GRN_COLUMN_FIX_SIZE ||
          obj->header.type == GRN_COLUMN_VAR_SIZE;
 }
 
-grn_bool
+/**
+ * \brief Check whether the specified object is an index column or not.
+ *
+ * \param ctx The context object
+ * \param obj The target object to check
+ *
+ * \return `true` if the object is exactly an index column, `false` otherwise.
+ */
+bool
 grn_obj_is_index_column(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_column(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   return obj->header.type == GRN_COLUMN_INDEX;
 }
 
-grn_bool
+bool
 grn_obj_is_accessor(grn_ctx *ctx, grn_obj *obj)
 {
   if (!obj) {
-    return GRN_FALSE;
+    return false;
   }
 
   return obj->header.type == GRN_ACCESSOR;
 }
 
-grn_bool
+bool
 grn_obj_is_id_accessor(grn_ctx *ctx, grn_obj *obj)
 {
   grn_accessor *accessor;
 
   if (!grn_obj_is_accessor(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   accessor = (grn_accessor *)obj;
   if (accessor->next) {
-    return GRN_FALSE;
+    return false;
   }
 
   return accessor->action == GRN_ACCESSOR_GET_ID;
 }
 
-grn_bool
+bool
 grn_obj_is_key_accessor(grn_ctx *ctx, grn_obj *obj)
 {
   grn_accessor *accessor;
 
   if (!grn_obj_is_accessor(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   accessor = (grn_accessor *)obj;
   if (accessor->next) {
-    return GRN_FALSE;
+    return false;
   }
 
   return accessor->action == GRN_ACCESSOR_GET_KEY;
@@ -629,12 +658,12 @@ grn_obj_is_value_accessor(grn_ctx *ctx, grn_obj *obj)
   grn_accessor *accessor;
 
   if (!grn_obj_is_accessor(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   accessor = (grn_accessor *)obj;
   if (accessor->next) {
-    return GRN_FALSE;
+    return false;
   }
 
   return accessor->action == GRN_ACCESSOR_GET_VALUE;
@@ -646,12 +675,12 @@ grn_obj_is_score_accessor(grn_ctx *ctx, grn_obj *obj)
   grn_accessor *accessor;
 
   if (!grn_obj_is_accessor(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   accessor = (grn_accessor *)obj;
   if (accessor->next) {
-    return GRN_FALSE;
+    return false;
   }
 
   return accessor->action == GRN_ACCESSOR_GET_SCORE;
@@ -688,12 +717,12 @@ grn_obj_is_nsubrecs_accessor(grn_ctx *ctx, grn_obj *obj)
   grn_accessor *accessor;
 
   if (!grn_obj_is_accessor(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   accessor = (grn_accessor *)obj;
   if (accessor->next) {
-    return GRN_FALSE;
+    return false;
   }
 
   return accessor->action == GRN_ACCESSOR_GET_NSUBRECS;
@@ -705,12 +734,12 @@ grn_obj_is_max_accessor(grn_ctx *ctx, grn_obj *obj)
   grn_accessor *accessor;
 
   if (!grn_obj_is_accessor(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   accessor = (grn_accessor *)obj;
   if (accessor->next) {
-    return GRN_FALSE;
+    return false;
   }
 
   return accessor->action == GRN_ACCESSOR_GET_MAX;
@@ -722,12 +751,12 @@ grn_obj_is_min_accessor(grn_ctx *ctx, grn_obj *obj)
   grn_accessor *accessor;
 
   if (!grn_obj_is_accessor(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   accessor = (grn_accessor *)obj;
   if (accessor->next) {
-    return GRN_FALSE;
+    return false;
   }
 
   return accessor->action == GRN_ACCESSOR_GET_MIN;
@@ -739,12 +768,12 @@ grn_obj_is_sum_accessor(grn_ctx *ctx, grn_obj *obj)
   grn_accessor *accessor;
 
   if (!grn_obj_is_accessor(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   accessor = (grn_accessor *)obj;
   if (accessor->next) {
-    return GRN_FALSE;
+    return false;
   }
 
   return accessor->action == GRN_ACCESSOR_GET_SUM;
@@ -756,12 +785,12 @@ grn_obj_is_avg_accessor(grn_ctx *ctx, grn_obj *obj)
   grn_accessor *accessor;
 
   if (!grn_obj_is_accessor(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   accessor = (grn_accessor *)obj;
   if (accessor->next) {
-    return GRN_FALSE;
+    return false;
   }
 
   return accessor->action == GRN_ACCESSOR_GET_AVG;
@@ -790,17 +819,83 @@ grn_obj_is_column_value_accessor(grn_ctx *ctx, grn_obj *obj)
   grn_accessor *accessor;
 
   if (!grn_obj_is_accessor(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   accessor = (grn_accessor *)obj;
   if (accessor->next) {
-    return GRN_FALSE;
+    return false;
   }
 
   return accessor->action == GRN_ACCESSOR_GET_COLUMN_VALUE;
 }
 
+bool
+grn_obj_is_scalar_accessor(grn_ctx *ctx, grn_obj *obj)
+{
+  grn_accessor *accessor;
+
+  if (!grn_obj_is_accessor(ctx, obj)) {
+    return false;
+  }
+
+  accessor = (grn_accessor *)obj;
+  while (accessor->next) {
+    accessor = accessor->next;
+  }
+
+  switch (accessor->action) {
+  case GRN_ACCESSOR_GET_ID:
+  case GRN_ACCESSOR_GET_SCORE:
+  case GRN_ACCESSOR_GET_NSUBRECS:
+  case GRN_ACCESSOR_GET_MAX:
+  case GRN_ACCESSOR_GET_MIN:
+  case GRN_ACCESSOR_GET_SUM:
+  case GRN_ACCESSOR_GET_AVG:
+  case GRN_ACCESSOR_GET_MEAN:
+  case GRN_ACCESSOR_GET_VALUE:
+    return true;
+  case GRN_ACCESSOR_GET_COLUMN_VALUE:
+    return grn_obj_is_scalar_column(ctx, accessor->obj);
+  default:
+    return false;
+  }
+}
+
+bool
+grn_obj_is_text_family_scalar_accessor(grn_ctx *ctx, grn_obj *obj)
+{
+  grn_accessor *accessor;
+
+  if (!grn_obj_is_accessor(ctx, obj)) {
+    return false;
+  }
+
+  accessor = (grn_accessor *)obj;
+  while (accessor->next) {
+    accessor = accessor->next;
+  }
+
+  switch (accessor->action) {
+  case GRN_ACCESSOR_GET_ID:
+  case GRN_ACCESSOR_GET_SCORE:
+  case GRN_ACCESSOR_GET_NSUBRECS:
+  case GRN_ACCESSOR_GET_MAX:
+  case GRN_ACCESSOR_GET_MIN:
+  case GRN_ACCESSOR_GET_SUM:
+  case GRN_ACCESSOR_GET_AVG:
+  case GRN_ACCESSOR_GET_MEAN:
+    return false;
+  case GRN_ACCESSOR_GET_VALUE:
+    return grn_type_id_is_text_family(ctx,
+                                      grn_obj_get_range(ctx, accessor->obj));
+  case GRN_ACCESSOR_GET_COLUMN_VALUE:
+    return grn_obj_is_text_family_scalar_column(ctx, accessor->obj);
+  default:
+    return false;
+  }
+}
+
 bool
 grn_obj_is_number_family_scalar_accessor(grn_ctx *ctx, grn_obj *obj)
 {
@@ -835,134 +930,178 @@ grn_obj_is_number_family_scalar_accessor(grn_ctx *ctx, grn_obj *obj)
   }
 }
 
-grn_bool
+bool
+grn_obj_is_vector_accessor(grn_ctx *ctx, grn_obj *obj)
+{
+  grn_accessor *accessor;
+
+  if (!grn_obj_is_accessor(ctx, obj)) {
+    return false;
+  }
+
+  accessor = (grn_accessor *)obj;
+  while (accessor->next) {
+    accessor = accessor->next;
+  }
+
+  switch (accessor->action) {
+  case GRN_ACCESSOR_GET_COLUMN_VALUE:
+    return grn_obj_is_vector_column(ctx, accessor->obj);
+  default:
+    return false;
+  }
+}
+
+bool
 grn_obj_is_type(grn_ctx *ctx, grn_obj *obj)
 {
   if (!obj) {
-    return GRN_FALSE;
+    return false;
   }
 
   return obj->header.type == GRN_TYPE;
 }
 
-grn_bool
+bool
 grn_obj_is_text_family_type(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_type(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   return GRN_TYPE_IS_TEXT_FAMILY(grn_obj_id(ctx, obj));
 }
 
-grn_bool
+bool
 grn_obj_is_proc(grn_ctx *ctx, grn_obj *obj)
 {
   if (!obj) {
-    return GRN_FALSE;
+    return false;
   }
 
   return obj->header.type == GRN_PROC;
 }
 
-grn_bool
+bool
 grn_obj_is_tokenizer_proc(grn_ctx *ctx, grn_obj *obj)
 {
   grn_proc *proc;
 
   if (!grn_obj_is_proc(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   proc = (grn_proc *)obj;
   return proc->type == GRN_PROC_TOKENIZER;
 }
 
-grn_bool
+bool
+grn_obj_is_command_proc(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!grn_obj_is_proc(ctx, obj)) {
+    return false;
+  }
+
+  grn_proc *proc = (grn_proc *)obj;
+  return proc->type == GRN_PROC_COMMAND;
+}
+
+bool
 grn_obj_is_function_proc(grn_ctx *ctx, grn_obj *obj)
 {
   grn_proc *proc;
 
   if (!grn_obj_is_proc(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   proc = (grn_proc *)obj;
   return proc->type == GRN_PROC_FUNCTION;
 }
 
-grn_bool
+bool
 grn_obj_is_selector_proc(grn_ctx *ctx, grn_obj *obj)
 {
   grn_proc *proc;
 
   if (!grn_obj_is_function_proc(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   proc = (grn_proc *)obj;
   return proc->callbacks.function.selector != NULL;
 }
 
-grn_bool
+bool
 grn_obj_is_selector_only_proc(grn_ctx *ctx, grn_obj *obj)
 {
   grn_proc *proc;
 
   if (!grn_obj_is_selector_proc(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   proc = (grn_proc *)obj;
   return proc->funcs[PROC_INIT] == NULL;
 }
 
-grn_bool
+bool
+grn_obj_is_applier_proc(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!grn_obj_is_function_proc(ctx, obj)) {
+    return false;
+  }
+
+  grn_proc *proc = (grn_proc *)obj;
+  return proc->callbacks.function.applier != NULL;
+}
+
+bool
 grn_obj_is_normalizer_proc(grn_ctx *ctx, grn_obj *obj)
 {
   grn_proc *proc;
 
   if (!grn_obj_is_proc(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   proc = (grn_proc *)obj;
   return proc->type == GRN_PROC_NORMALIZER;
 }
 
-grn_bool
+bool
 grn_obj_is_token_filter_proc(grn_ctx *ctx, grn_obj *obj)
 {
   grn_proc *proc;
 
   if (!grn_obj_is_proc(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   proc = (grn_proc *)obj;
   return proc->type == GRN_PROC_TOKEN_FILTER;
 }
 
-grn_bool
+bool
 grn_obj_is_scorer_proc(grn_ctx *ctx, grn_obj *obj)
 {
   grn_proc *proc;
 
   if (!grn_obj_is_proc(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   proc = (grn_proc *)obj;
   return proc->type == GRN_PROC_SCORER;
 }
 
-grn_bool
+bool
 grn_obj_is_window_function_proc(grn_ctx *ctx, grn_obj *obj)
 {
   grn_proc *proc;
 
   if (!grn_obj_is_proc(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   proc = (grn_proc *)obj;
@@ -982,11 +1121,11 @@ grn_obj_is_aggregator_proc(grn_ctx *ctx, grn_obj *obj)
   return proc->type == GRN_PROC_AGGREGATOR;
 }
 
-grn_bool
+bool
 grn_obj_is_expr(grn_ctx *ctx, grn_obj *obj)
 {
   if (!obj) {
-    return GRN_FALSE;
+    return false;
   }
 
   return obj->header.type == GRN_EXPR;
@@ -1041,13 +1180,14 @@ grn_obj_set_visibility(grn_ctx *ctx, grn_obj *obj, bool is_visible)
   }
 
   if (!(grn_obj_is_index_column(ctx, obj) ||
-        grn_obj_is_token_column(ctx, obj))) {
+        grn_obj_is_token_column(ctx, obj) ||
+        grn_obj_is_generated_column(ctx, obj))) {
     grn_obj inspected;
     GRN_TEXT_INIT(&inspected, 0);
     grn_inspect_limited(ctx, &inspected, obj);
     ERR(GRN_INVALID_ARGUMENT,
         "[obj][set-visibility] "
-        "must be an index column or a token column: <%.*s>",
+        "must be an index column, a token column or a generated column: <%.*s>",
         (int)(GRN_TEXT_LEN(&inspected)),
         GRN_TEXT_VALUE(&inspected));
     GRN_OBJ_FIN(ctx, &inspected);
@@ -1080,7 +1220,33 @@ grn_obj_have_source(grn_ctx *ctx, grn_obj *obj)
 bool
 grn_obj_is_token_column(grn_ctx *ctx, grn_obj *obj)
 {
-  return grn_obj_is_vector_column(ctx, obj) && grn_obj_have_source(ctx, obj);
+  return grn_obj_is_vector_column(ctx, obj) && grn_obj_have_source(ctx, obj) &&
+         ((grn_ja *)obj)->generator.length == 0;
+}
+
+bool
+grn_obj_is_generated_column(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!obj) {
+    return false;
+  }
+
+  grn_raw_string *generator = NULL;
+  switch (obj->header.type) {
+  case GRN_COLUMN_FIX_SIZE:
+    generator = &(((grn_ra *)obj)->generator);
+    break;
+  case GRN_COLUMN_VAR_SIZE:
+    generator = &(((grn_ja *)obj)->generator);
+    break;
+  default:
+    return false;
+  }
+  if (!grn_obj_have_source(ctx, obj)) {
+    return false;
+  }
+
+  return generator->length > 0;
 }
 
 static void
@@ -1288,11 +1454,38 @@ grn_obj_type_to_string(uint8_t type)
   }
 }
 
-grn_bool
+bool
+grn_obj_type_is_table(uint8_t type)
+{
+  switch (type) {
+  case GRN_TABLE_HASH_KEY:
+  case GRN_TABLE_PAT_KEY:
+  case GRN_TABLE_DAT_KEY:
+  case GRN_TABLE_NO_KEY:
+    return true;
+  default:
+    return false;
+  }
+}
+
+bool
+grn_obj_type_is_column(uint8_t type)
+{
+  switch (type) {
+  case GRN_COLUMN_FIX_SIZE:
+  case GRN_COLUMN_VAR_SIZE:
+  case GRN_COLUMN_INDEX:
+    return true;
+  default:
+    return false;
+  }
+}
+
+bool
 grn_obj_name_is_column(grn_ctx *ctx, const char *name, int name_len)
 {
   if (!name) {
-    return GRN_FALSE;
+    return false;
   }
 
   if (name_len < 0) {
@@ -1302,6 +1495,18 @@ grn_obj_name_is_column(grn_ctx *ctx, const char *name, int name_len)
   return memchr(name, GRN_DB_DELIMITER, (size_t)name_len) != NULL;
 }
 
+bool
+grn_obj_is_empty(grn_ctx *ctx, grn_obj *obj, grn_id id)
+{
+  if (obj->header.type == GRN_COLUMN_VAR_SIZE) {
+    return grn_ja_is_empty(ctx, (grn_ja *)obj, id);
+  }
+  ERR(GRN_FUNCTION_NOT_IMPLEMENTED,
+      "[obj][is-empty] unsupported type: %s",
+      grn_obj_type_to_string(obj->header.type));
+  return false;
+}
+
 grn_io *
 grn_obj_get_io(grn_ctx *ctx, grn_obj *obj)
 {
