@@ -1,10 +1,11 @@
 /*
   Copyright (C) 2009-2018  Brazil
-  Copyright (C) 2018-2022  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2018-2024  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -45,7 +46,8 @@ grn_token_cursor_open_initialize_token_filters(grn_ctx *ctx,
     return;
   }
 
-  token_cursor->token_filter.data = GRN_CALLOC(sizeof(void *) * n_token_filters);
+  token_cursor->token_filter.data =
+    GRN_CALLOC(sizeof(void *) * n_token_filters);
   if (!token_cursor->token_filter.data) {
     return;
   }
@@ -72,8 +74,10 @@ grn_token_cursor_open_initialize_token_filters(grn_ctx *ctx,
 }
 
 grn_token_cursor *
-grn_token_cursor_open(grn_ctx *ctx, grn_obj *table,
-                      const char *str, size_t str_len,
+grn_token_cursor_open(grn_ctx *ctx,
+                      grn_obj *table,
+                      const char *str,
+                      size_t str_len,
                       grn_tokenize_mode mode,
                       uint32_t flags)
 {
@@ -84,8 +88,13 @@ grn_token_cursor_open(grn_ctx *ctx, grn_obj *table,
   grn_obj *tokenizer;
   grn_obj *token_filters;
   grn_table_flags table_flags;
-  if (grn_table_get_info(ctx, table, &table_flags, &encoding, &tokenizer,
-                         NULL, &token_filters)) {
+  if (grn_table_get_info(ctx,
+                         table,
+                         &table_flags,
+                         &encoding,
+                         &tokenizer,
+                         NULL,
+                         &token_filters)) {
     ERR(GRN_INVALID_ARGUMENT,
         "[token-cursor][open] failed to get table information");
     GRN_API_RETURN(NULL);
@@ -103,6 +112,11 @@ grn_token_cursor_open(grn_ctx *ctx, grn_obj *table,
   grn_tokenizer_query_set_lexicon(ctx, &(token_cursor->tokenizer.query), table);
   grn_tokenizer_query_set_flags(ctx, &(token_cursor->tokenizer.query), flags);
   grn_tokenizer_query_set_mode(ctx, &(token_cursor->tokenizer.query), mode);
+  token_cursor->tokenizer.query_domain = GRN_ID_NIL;
+  GRN_OBJ_INIT(&(token_cursor->tokenizer.casted_query),
+               GRN_BULK,
+               0,
+               table->header.domain);
   grn_token_init(ctx, &(token_cursor->tokenizer.current_token));
   grn_token_init(ctx, &(token_cursor->tokenizer.next_token));
   grn_token_init(ctx, &(token_cursor->tokenizer.original_token));
@@ -120,9 +134,18 @@ grn_token_cursor_open(grn_ctx *ctx, grn_obj *table,
   GRN_API_RETURN(token_cursor);
 }
 
+grn_rc
+grn_token_cursor_set_query_domain(grn_ctx *ctx,
+                                  grn_token_cursor *token_cursor,
+                                  grn_id domain)
+{
+  GRN_API_ENTER;
+  token_cursor->tokenizer.query_domain = domain;
+  GRN_API_RETURN(ctx->rc);
+}
+
 static bool
-grn_token_cursor_ensure_initialize(grn_ctx *ctx,
-                                   grn_token_cursor *token_cursor)
+grn_token_cursor_ensure_initialize(grn_ctx *ctx, grn_token_cursor *token_cursor)
 {
   if (token_cursor->initialized) {
     goto exit;
@@ -149,11 +172,16 @@ grn_token_cursor_ensure_initialize(grn_ctx *ctx,
   if (token_cursor->tokenizer.object) {
     grn_proc *tokenizer_proc = (grn_proc *)(token_cursor->tokenizer.object);
     if (tokenizer_proc->callbacks.tokenizer.init) {
+      grn_id query_domain = token_cursor->tokenizer.query_domain;
+      if (query_domain == GRN_ID_NIL) {
+        query_domain = GRN_DB_TEXT;
+      }
       token_cursor->tokenizer.user_data = NULL;
-      grn_tokenizer_query_set_raw_string(ctx,
-                                         query,
-                                         token_cursor->orig,
-                                         token_cursor->orig_blen);
+      grn_tokenizer_query_set_data(ctx,
+                                   query,
+                                   token_cursor->orig,
+                                   token_cursor->orig_blen,
+                                   query_domain);
       if (ctx->rc != GRN_SUCCESS) {
         goto exit;
       }
@@ -162,9 +190,7 @@ grn_token_cursor_ensure_initialize(grn_ctx *ctx,
     } else if (tokenizer_proc->funcs[PROC_INIT]) {
       grn_obj str_, flags_, mode_;
       GRN_TEXT_INIT(&str_, GRN_OBJ_DO_SHALLOW_COPY);
-      GRN_TEXT_SET_REF(&str_,
-                       token_cursor->orig,
-                       token_cursor->orig_blen);
+      GRN_TEXT_SET_REF(&str_, token_cursor->orig, token_cursor->orig_blen);
       GRN_UINT32_INIT(&flags_, 0);
       GRN_UINT32_SET(ctx, &flags_, token_cursor->flags);
       GRN_UINT32_INIT(&mode_, 0);
@@ -178,36 +204,78 @@ grn_token_cursor_ensure_initialize(grn_ctx *ctx,
       grn_ctx_push(ctx, &mode_);
       grn_ctx_push(ctx, &str_);
       grn_ctx_push(ctx, &flags_);
-      tokenizer_proc->funcs[PROC_INIT](ctx,
-                                       1,
-                                       &(token_cursor->table),
-                                       &(token_cursor->tokenizer.pctx.user_data));
+      tokenizer_proc->funcs[PROC_INIT](
+        ctx,
+        1,
+        &(token_cursor->table),
+        &(token_cursor->tokenizer.pctx.user_data));
       grn_obj_close(ctx, &flags_);
       grn_obj_close(ctx, &str_);
       grn_obj_close(ctx, &mode_);
     }
   } else {
-    grn_obj *string;
-    const char *normalized;
+    grn_id query_domain = token_cursor->tokenizer.query_domain;
 
-    grn_tokenizer_query_set_raw_string(ctx,
-                                       query,
-                                       token_cursor->orig,
-                                       token_cursor->orig_blen);
+    grn_tokenizer_query_set_data(ctx,
+                                 query,
+                                 token_cursor->orig,
+                                 token_cursor->orig_blen,
+                                 query_domain == GRN_ID_NIL ? GRN_DB_TEXT
+                                                            : query_domain);
     if (ctx->rc != GRN_SUCCESS) {
       goto exit;
     }
-    string = grn_tokenizer_query_get_normalized_string(ctx, query);
-    grn_string_get_normalized(ctx,
-                              string,
-                              &normalized,
-                              &(token_cursor->curr_size),
-                              NULL);
-    token_cursor->curr = (const unsigned char *)normalized;
-    grn_token_set_data(ctx,
-                       &(token_cursor->tokenizer.current_token),
-                       token_cursor->curr,
-                       (int)(token_cursor->curr_size));
+    grn_obj *casted_query = &(token_cursor->tokenizer.casted_query);
+    grn_obj *table = token_cursor->table;
+    grn_id table_id = casted_query->header.domain;
+    if (query_domain == GRN_ID_NIL ||
+        (grn_type_id_is_text_family(ctx, query_domain) &&
+         grn_type_id_is_text_family(ctx, casted_query->header.domain))) {
+      grn_obj *string;
+      const char *normalized;
+      string = grn_tokenizer_query_get_normalized_string(ctx, query);
+      grn_string_get_normalized(ctx,
+                                string,
+                                &normalized,
+                                &(token_cursor->curr_size),
+                                NULL);
+      token_cursor->curr = (const unsigned char *)normalized;
+      grn_token_set_data(ctx,
+                         &(token_cursor->tokenizer.current_token),
+                         token_cursor->curr,
+                         (int)(token_cursor->curr_size));
+    } else {
+      if (query_domain == table_id || query_domain == table->header.domain) {
+        token_cursor->curr = token_cursor->orig;
+        token_cursor->curr_size = token_cursor->orig_blen;
+        grn_token_set_data(ctx,
+                           &(token_cursor->tokenizer.current_token),
+                           token_cursor->curr,
+                           (int)(token_cursor->curr_size));
+        grn_token_set_domain(ctx,
+                             &(token_cursor->tokenizer.current_token),
+                             query_domain);
+      } else {
+        grn_obj query;
+        GRN_OBJ_INIT(&query, GRN_BULK, GRN_OBJ_DO_SHALLOW_COPY, query_domain);
+        GRN_TEXT_SET(ctx, &query, token_cursor->orig, token_cursor->orig_blen);
+        GRN_BULK_REWIND(casted_query);
+        grn_rc rc = grn_obj_cast(ctx, &query, casted_query, false);
+        GRN_OBJ_FIN(ctx, &query);
+        if (rc == GRN_SUCCESS) {
+          token_cursor->curr = GRN_BULK_HEAD(casted_query);
+          token_cursor->curr_size = GRN_BULK_VSIZE(casted_query);
+          grn_token_set_data(ctx,
+                             &(token_cursor->tokenizer.current_token),
+                             token_cursor->curr,
+                             (int)(token_cursor->curr_size));
+          grn_token_set_domain(
+            ctx,
+            &(token_cursor->tokenizer.current_token),
+            token_cursor->tokenizer.casted_query.header.domain);
+        }
+      }
+    }
     grn_token_set_status(ctx,
                          &(token_cursor->tokenizer.current_token),
                          GRN_TOKEN_LAST);
@@ -217,7 +285,7 @@ grn_token_cursor_ensure_initialize(grn_ctx *ctx,
     grn_token_cursor_open_initialize_token_filters(ctx, token_cursor);
   }
 
-exit :
+exit:
   return ctx->rc == GRN_SUCCESS;
 }
 
@@ -293,9 +361,7 @@ grn_token_cursor_next_apply_token_filters(grn_ctx *ctx,
 
       grn_tokenizer_query_set_token_filter_index(ctx, query, (unsigned int)i);
 
-#define SKIP_FLAGS                              \
-      (GRN_TOKEN_SKIP |                         \
-       GRN_TOKEN_SKIP_WITH_POSITION)
+#define SKIP_FLAGS (GRN_TOKEN_SKIP | GRN_TOKEN_SKIP_WITH_POSITION)
       if (grn_token_get_status(ctx, current_token) & SKIP_FLAGS) {
         break;
       }
@@ -328,13 +394,11 @@ grn_token_cursor_next_need_keep_original(grn_ctx *ctx,
   }
 
   grn_raw_string current_data;
-  current_data.value = grn_token_get_data_raw(ctx,
-                                              current_token,
-                                              &(current_data.length));
+  current_data.value =
+    grn_token_get_data_raw(ctx, current_token, &(current_data.length));
   grn_raw_string original_data;
-  original_data.value = grn_token_get_data_raw(ctx,
-                                               original_token,
-                                               &(original_data.length));
+  original_data.value =
+    grn_token_get_data_raw(ctx, original_token, &(original_data.length));
   return !GRN_RAW_STRING_EQUAL(current_data, original_data);
 }
 
@@ -358,9 +422,7 @@ grn_token_cursor_next(grn_ctx *ctx, grn_token_cursor *token_cursor)
     if (token_cursor->pending.token) {
       token_cursor->status = token_cursor->pending.status;
       grn_token_copy(ctx, current_token, token_cursor->pending.token);
-      grn_token_remove_status(ctx,
-                              current_token,
-                              GRN_TOKEN_KEEP_ORIGINAL);
+      grn_token_remove_status(ctx, current_token, GRN_TOKEN_KEEP_ORIGINAL);
       size_t size;
       token_cursor->curr = grn_token_get_data_raw(ctx, current_token, &size);
       token_cursor->curr_size = (uint32_t)size;
@@ -376,10 +438,11 @@ grn_token_cursor_next(grn_ctx *ctx, grn_token_cursor *token_cursor)
                                                  user_data);
       } else if (tokenizer_proc->funcs[PROC_NEXT]) {
         grn_obj *data, *status_obj;
-        tokenizer_proc->funcs[PROC_NEXT](ctx,
-                                         1,
-                                         &table,
-                                         &token_cursor->tokenizer.pctx.user_data);
+        tokenizer_proc->funcs[PROC_NEXT](
+          ctx,
+          1,
+          &table,
+          &token_cursor->tokenizer.pctx.user_data);
         status_obj = grn_ctx_pop(ctx);
         data = grn_ctx_pop(ctx);
         grn_token_set_data(ctx,
@@ -396,17 +459,17 @@ grn_token_cursor_next(grn_ctx *ctx, grn_token_cursor *token_cursor)
     {
       grn_token_status status = grn_token_get_status(ctx, current_token);
       token_cursor->status =
-        ((status & GRN_TOKEN_LAST) ||
-         (token_cursor->mode == GRN_TOKENIZE_GET &&
-          (status & GRN_TOKEN_REACH_END)))
-        ? GRN_TOKEN_CURSOR_DONE : GRN_TOKEN_CURSOR_DOING;
-#define SKIP_FLAGS \
-      (GRN_TOKEN_SKIP | GRN_TOKEN_SKIP_WITH_POSITION)
+        ((status & GRN_TOKEN_LAST) || (token_cursor->mode == GRN_TOKENIZE_GET &&
+                                       (status & GRN_TOKEN_REACH_END)))
+          ? GRN_TOKEN_CURSOR_DONE
+          : GRN_TOKEN_CURSOR_DOING;
+#define SKIP_FLAGS (GRN_TOKEN_SKIP | GRN_TOKEN_SKIP_WITH_POSITION)
       if (status & SKIP_FLAGS) {
         if (status & GRN_TOKEN_SKIP) {
           token_cursor->pos++;
         }
-        if (token_cursor->status == GRN_TOKEN_CURSOR_DONE && tid == GRN_ID_NIL) {
+        if (token_cursor->status == GRN_TOKEN_CURSOR_DONE &&
+            tid == GRN_ID_NIL) {
           token_cursor->status = GRN_TOKEN_CURSOR_DONE_SKIP;
           break;
         } else {
@@ -415,18 +478,20 @@ grn_token_cursor_next(grn_ctx *ctx, grn_token_cursor *token_cursor)
       }
 #undef SKIP_FLAGS
       if (status & GRN_TOKEN_FORCE_PREFIX) {
-        grn_token_set_force_prefix_search(ctx, current_token, GRN_TRUE);
+        grn_token_set_force_prefix_search(ctx, current_token, true);
       }
       if (token_cursor->curr_size == 0) {
         continue;
       }
       if (token_cursor->curr_size > GRN_TABLE_MAX_KEY_SIZE) {
-        GRN_LOG(ctx, GRN_WARN,
+        GRN_LOG(ctx,
+                GRN_WARN,
                 "[token_next] ignore too long token. "
                 "Token must be less than or equal to %d: <%d>(<%.*s>)",
                 GRN_TABLE_MAX_KEY_SIZE,
                 token_cursor->curr_size,
-                token_cursor->curr_size, token_cursor->curr);
+                token_cursor->curr_size,
+                token_cursor->curr);
         continue;
       }
       if (status & GRN_TOKEN_UNMATURED) {
@@ -437,14 +502,17 @@ grn_token_cursor_next(grn_ctx *ctx, grn_token_cursor *token_cursor)
           }
         } else {
           if (status & GRN_TOKEN_REACH_END) {
-            grn_token_set_force_prefix_search(ctx, current_token, GRN_TRUE);
+            grn_token_set_force_prefix_search(ctx, current_token, true);
           }
         }
       }
     }
-    if (token_cursor->mode == GRN_TOKENIZE_ADD) {
+    if (grn_id_maybe_table(ctx, grn_token_get_domain(ctx, current_token)) &&
+        GRN_BULK_VSIZE(&(current_token->data)) == sizeof(grn_id)) {
+      tid = GRN_RECORD_VALUE(&(current_token->data));
+    } else if (token_cursor->mode == GRN_TOKENIZE_ADD) {
       switch (table->header.type) {
-      case GRN_TABLE_PAT_KEY :
+      case GRN_TABLE_PAT_KEY:
         if (token_cursor->flags & GRN_TOKEN_CURSOR_PARALLEL) {
           tid = grn_pat_get(ctx,
                             (grn_pat *)table,
@@ -453,17 +521,19 @@ grn_token_cursor_next(grn_ctx *ctx, grn_token_cursor *token_cursor)
                             NULL);
         }
         if (tid == GRN_ID_NIL) {
-          GRN_TABLE_LOCK_BEGIN(ctx, table) {
+          GRN_TABLE_LOCK_BEGIN(ctx, table)
+          {
             tid = grn_pat_add(ctx,
                               (grn_pat *)table,
                               token_cursor->curr,
                               token_cursor->curr_size,
                               NULL,
                               NULL);
-          } GRN_TABLE_LOCK_END(ctx);
+          }
+          GRN_TABLE_LOCK_END(ctx);
         }
         break;
-      case GRN_TABLE_DAT_KEY :
+      case GRN_TABLE_DAT_KEY:
         if (token_cursor->flags & GRN_TOKEN_CURSOR_PARALLEL) {
           tid = grn_dat_get(ctx,
                             (grn_dat *)table,
@@ -472,17 +542,19 @@ grn_token_cursor_next(grn_ctx *ctx, grn_token_cursor *token_cursor)
                             NULL);
         }
         if (tid == GRN_ID_NIL) {
-          GRN_TABLE_LOCK_BEGIN(ctx, table) {
+          GRN_TABLE_LOCK_BEGIN(ctx, table)
+          {
             tid = grn_dat_add(ctx,
                               (grn_dat *)table,
                               token_cursor->curr,
                               token_cursor->curr_size,
                               NULL,
                               NULL);
-          } GRN_TABLE_LOCK_END(ctx);
+          }
+          GRN_TABLE_LOCK_END(ctx);
         }
         break;
-      case GRN_TABLE_HASH_KEY :
+      case GRN_TABLE_HASH_KEY:
         if (token_cursor->flags & GRN_TOKEN_CURSOR_PARALLEL) {
           tid = grn_hash_get(ctx,
                              (grn_hash *)table,
@@ -490,16 +562,18 @@ grn_token_cursor_next(grn_ctx *ctx, grn_token_cursor *token_cursor)
                              token_cursor->curr_size,
                              NULL);
         }
-        GRN_TABLE_LOCK_BEGIN(ctx, table) {
+        GRN_TABLE_LOCK_BEGIN(ctx, table)
+        {
           tid = grn_hash_add(ctx,
                              (grn_hash *)table,
                              token_cursor->curr,
                              token_cursor->curr_size,
                              NULL,
                              NULL);
-        } GRN_TABLE_LOCK_END(ctx);
+        }
+        GRN_TABLE_LOCK_END(ctx);
         break;
-      case GRN_TABLE_NO_KEY :
+      case GRN_TABLE_NO_KEY:
         if (token_cursor->curr_size == sizeof(grn_id)) {
           tid = *((grn_id *)token_cursor->curr);
         } else {
@@ -509,16 +583,28 @@ grn_token_cursor_next(grn_ctx *ctx, grn_token_cursor *token_cursor)
       }
     } else if (token_cursor->mode != GRN_TOKENIZE_ONLY) {
       switch (table->header.type) {
-      case GRN_TABLE_PAT_KEY :
-        tid = grn_pat_get(ctx, (grn_pat *)table, token_cursor->curr, token_cursor->curr_size, NULL);
+      case GRN_TABLE_PAT_KEY:
+        tid = grn_pat_get(ctx,
+                          (grn_pat *)table,
+                          token_cursor->curr,
+                          token_cursor->curr_size,
+                          NULL);
         break;
-      case GRN_TABLE_DAT_KEY :
-        tid = grn_dat_get(ctx, (grn_dat *)table, token_cursor->curr, token_cursor->curr_size, NULL);
+      case GRN_TABLE_DAT_KEY:
+        tid = grn_dat_get(ctx,
+                          (grn_dat *)table,
+                          token_cursor->curr,
+                          token_cursor->curr_size,
+                          NULL);
         break;
-      case GRN_TABLE_HASH_KEY :
-        tid = grn_hash_get(ctx, (grn_hash *)table, token_cursor->curr, token_cursor->curr_size, NULL);
+      case GRN_TABLE_HASH_KEY:
+        tid = grn_hash_get(ctx,
+                           (grn_hash *)table,
+                           token_cursor->curr,
+                           token_cursor->curr_size,
+                           NULL);
         break;
-      case GRN_TABLE_NO_KEY :
+      case GRN_TABLE_NO_KEY:
         if (token_cursor->curr_size == sizeof(grn_id)) {
           tid = *((grn_id *)token_cursor->curr);
         } else {
@@ -527,8 +613,8 @@ grn_token_cursor_next(grn_ctx *ctx, grn_token_cursor *token_cursor)
         break;
       }
     }
-    if (token_cursor->mode != GRN_TOKENIZE_ONLY &&
-        tid == GRN_ID_NIL && token_cursor->status != GRN_TOKEN_CURSOR_DONE) {
+    if (token_cursor->mode != GRN_TOKENIZE_ONLY && tid == GRN_ID_NIL &&
+        token_cursor->status != GRN_TOKEN_CURSOR_DONE) {
       token_cursor->status = GRN_TOKEN_CURSOR_NOT_FOUND;
     }
     if (grn_token_cursor_next_need_keep_original(ctx, token_cursor)) {
@@ -614,6 +700,7 @@ grn_token_cursor_close(grn_ctx *ctx, grn_token_cursor *token_cursor)
   grn_token_fin(ctx, &(token_cursor->tokenizer.current_token));
   grn_token_fin(ctx, &(token_cursor->tokenizer.next_token));
   grn_token_fin(ctx, &(token_cursor->tokenizer.original_token));
+  GRN_OBJ_FIN(ctx, &(token_cursor->tokenizer.casted_query));
   grn_tokenizer_query_fin(ctx, &(token_cursor->tokenizer.query));
   grn_token_cursor_close_token_filters(ctx, token_cursor);
   GRN_OBJ_FIN(ctx, &(token_cursor->original_buffer));
