@@ -4,7 +4,8 @@
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -51,18 +52,18 @@ grn_tokenizer_isspace(grn_ctx *ctx,
   return grn_plugin_isspace(ctx, str_ptr, str_length, encoding);
 }
 
-grn_bool
+bool
 grn_tokenizer_is_tokenized_delimiter(grn_ctx *ctx,
                                      const char *str_ptr,
                                      unsigned int str_length,
                                      grn_encoding encoding)
 {
   if (encoding != GRN_ENC_UTF8) {
-    return GRN_FALSE;
+    return false;
   }
 
   if (str_length != GRN_TOKENIZER_TOKENIZED_DELIMITER_UTF8_LEN) {
-    return GRN_FALSE;
+    return false;
   }
 
   return memcmp(str_ptr,
@@ -70,7 +71,7 @@ grn_tokenizer_is_tokenized_delimiter(grn_ctx *ctx,
                 GRN_TOKENIZER_TOKENIZED_DELIMITER_UTF8_LEN) == 0;
 }
 
-grn_bool
+bool
 grn_tokenizer_have_tokenized_delimiter(grn_ctx *ctx,
                                        const char *str_ptr,
                                        unsigned int str_length,
@@ -81,11 +82,11 @@ grn_tokenizer_have_tokenized_delimiter(grn_ctx *ctx,
   const char *end = str_ptr + str_length;
 
   if (encoding != GRN_ENC_UTF8) {
-    return GRN_FALSE;
+    return false;
   }
 
   if (str_length == 0) {
-    return GRN_FALSE;
+    return false;
   }
 
   while ((char_length = grn_charlen_(ctx, current, end, encoding)) > 0) {
@@ -93,11 +94,11 @@ grn_tokenizer_have_tokenized_delimiter(grn_ctx *ctx,
                                              current,
                                              (unsigned int)char_length,
                                              encoding)) {
-      return GRN_TRUE;
+      return true;
     }
     current += char_length;
   }
-  return GRN_FALSE;
+  return false;
 }
 
 static void
@@ -107,7 +108,7 @@ grn_tokenizer_query_ensure_normalized(grn_ctx *ctx, grn_tokenizer_query *query)
     return;
   }
 
-  query->need_normalize = GRN_FALSE;
+  query->need_normalize = false;
 
   if (query->normalized_query) {
     grn_obj_close(ctx, query->normalized_query);
@@ -119,7 +120,7 @@ grn_tokenizer_query_ensure_normalized(grn_ctx *ctx, grn_tokenizer_query *query)
                                              (int)(query->normalize_flags),
                                              query->encoding);
   if (!query->normalized_query) {
-    query->have_tokenized_delimiter = GRN_FALSE;
+    query->have_tokenized_delimiter = false;
     GRN_PLUGIN_ERROR(ctx,
                      GRN_TOKENIZER_ERROR,
                      "[tokenizer][normalize] failed to open normalized string: "
@@ -130,7 +131,7 @@ grn_tokenizer_query_ensure_normalized(grn_ctx *ctx, grn_tokenizer_query *query)
     return;
   }
 
-  query->need_delimiter_check = GRN_TRUE;
+  query->need_delimiter_check = true;
 }
 
 static void
@@ -143,7 +144,7 @@ grn_tokenizer_query_ensure_have_tokenized_delimiter(grn_ctx *ctx,
     return;
   }
 
-  query->need_delimiter_check = GRN_FALSE;
+  query->need_delimiter_check = false;
 
   if (query->flags & GRN_TOKEN_CURSOR_ENABLE_TOKENIZED_DELIMITER) {
     const char *normalized_string;
@@ -160,7 +161,7 @@ grn_tokenizer_query_ensure_have_tokenized_delimiter(grn_ctx *ctx,
                                              normalized_string_length,
                                              query->encoding);
   } else {
-    query->have_tokenized_delimiter = GRN_FALSE;
+    query->have_tokenized_delimiter = false;
   }
 }
 
@@ -182,8 +183,8 @@ grn_tokenizer_query_init(grn_ctx *ctx, grn_tokenizer_query *query)
   query->source_id = GRN_ID_NIL;
   query->index_column = NULL;
 
-  query->need_normalize = GRN_TRUE;
-  query->need_delimiter_check = GRN_TRUE;
+  query->need_normalize = true;
+  query->need_delimiter_check = true;
 
   query->options = NULL;
 
@@ -290,7 +291,7 @@ grn_tokenizer_query_set_normalize_flags(grn_ctx *ctx,
   GRN_API_ENTER;
   if (query->normalize_flags != normalize_flags) {
     query->normalize_flags = normalize_flags;
-    query->need_normalize = GRN_TRUE;
+    query->need_normalize = true;
   }
   GRN_API_RETURN(ctx->rc);
 }
@@ -377,7 +378,7 @@ grn_tokenizer_query_set_data(grn_ctx *ctx,
   query->size = size;
   query->ptr = query->data;
   query->domain = domain;
-  query->need_normalize = GRN_TRUE;
+  query->need_normalize = true;
 
   GRN_API_RETURN(ctx->rc);
 }
@@ -413,7 +414,7 @@ grn_tokenizer_query_set_flags(grn_ctx *ctx,
   GRN_API_ENTER;
   if (query->flags != flags) {
     query->flags = flags;
-    query->need_normalize = GRN_TRUE;
+    query->need_normalize = true;
   }
   GRN_API_RETURN(ctx->rc);
 }
@@ -425,7 +426,7 @@ grn_tokenizer_query_get_flags(grn_ctx *ctx, grn_tokenizer_query *query)
   GRN_API_RETURN(query->flags);
 }
 
-grn_bool
+bool
 grn_tokenizer_query_have_tokenized_delimiter(grn_ctx *ctx,
                                              grn_tokenizer_query *query)
 {
@@ -443,7 +444,7 @@ grn_tokenizer_query_set_mode(grn_ctx *ctx,
   if (query->tokenize_mode != mode) {
     query->tokenize_mode = mode;
     query->token_mode = query->tokenize_mode;
-    query->need_normalize = GRN_TRUE;
+    query->need_normalize = true;
   }
   GRN_API_RETURN(ctx->rc);
 }
@@ -475,7 +476,7 @@ grn_tokenizer_query_set_lexicon(grn_ctx *ctx,
     } else {
       query->encoding = ctx->encoding;
     }
-    query->need_normalize = GRN_TRUE;
+    query->need_normalize = true;
   }
 
   GRN_API_RETURN(ctx->rc);
