@@ -4,7 +4,8 @@
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -79,7 +80,7 @@ typedef struct {
   const char *log_base_path;
   FILE *log_file;
   uint32_t log_count;
-  grn_bool request_reopen_log_file;
+  bool request_reopen_log_file;
 } thd_data;
 
 typedef struct {
@@ -291,7 +292,7 @@ close_log_file(thd_data *thread)
 {
   fclose(thread->log_file);
   thread->log_file = NULL;
-  thread->request_reopen_log_file = GRN_FALSE;
+  thread->request_reopen_log_file = false;
 }
 
 static void
@@ -410,7 +411,7 @@ signal_reopen_log_file(int sig)
   uint32_t i;
 
   for (i = 0; i < max_threads; i++) {
-    threads[i].request_reopen_log_file = GRN_TRUE;
+    threads[i].request_reopen_log_file = true;
   }
 }
 
@@ -606,7 +607,7 @@ serve_threads(uint32_t nthreads, uint16_t port, const char *db_path, void *zmq_c
 
   for (i = 0; i < nthreads; i++) {
     memset(&threads[i], 0, sizeof(threads[i]));
-    threads[i].request_reopen_log_file = GRN_FALSE;
+    threads[i].request_reopen_log_file = false;
     if (!(threads[i].base = event_init())) {
       print_error("error in event_init() on thread %d.", i);
     } else {
